import java.awt.print.Book;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Map;
import java.util.Vector;

import Rabin.datastructures.BidiSortedMap;
import Rabin.datastructures.Interval;
import Rabin.fingerprint.RabinFingerprintLong;
import Rabin.handprint.FingerFactory;
import Rabin.handprint.HandPrint;
import Rabin.polynomial.Polynomial;


public class CalcFingerPrintOfFiles implements Runnable{
	public DataOutputStream out;
	public Vector<String> fileAddress = new Vector<String>();
	public Polynomial polynomial; //Polynomial.createIrreducible( 53 );
	public FingerFactory factory; 
	public byte[] buffer = new byte[65536];
	RabinFingerprintLong rabin; 
	public long processedDataSize = 0;
	
	int threadNum = -1;
	public CalcFingerPrintOfFiles(int threadNum, DataOutputStream out){
		try{
//			FileOutputStream fos = new FileOutputStream("polynomial.dat");
//			ObjectOutputStream oos = new ObjectOutputStream(fos);
//			oos.writeObject(polynomial);
//			oos.close();
			
			FileInputStream fis = new FileInputStream("polynomial.dat");
			ObjectInputStream ois = new ObjectInputStream(fis);
			polynomial = (Polynomial) ois.readObject();
			ois.close();
			
			factory = new FingerFactory( polynomial, FingerFactory.WINDOW_SIZE );
			rabin = new RabinFingerprintLong(polynomial);
			this.out = out;
			this.threadNum = threadNum;
		}catch (Exception e) {
			BLLogger.logData(e.getMessage());
		}
	}
	
	public void run(){
		
		try {
			calcFingerPrint();
		//	System.out.println("thread " + threadNum + " has finished at "+new Date());
			Thread.currentThread().stop();
		} catch (Exception e) {
			BLLogger.logData(e.getMessage());
		}
	}

	public void calcFingerPrint() {
		for (int i = 0; i < fileAddress.size(); i++) {
			try {
			//	System.out.println(fileAddress.get(i));
				getFingerPrints(new File(fileAddress.get(i)));
			} catch (Exception ex) {
				BLLogger.logData(ex.getMessage());
			}
		}

		fileAddress.clear();
	//	Thread.currentThread().stop();
	}
	
	private void getFingerPrints(File file) throws IOException {
		
		processedDataSize += file.length();
		HandPrint hand1 = new HandPrint( file, factory );
		//final Iterator<Long> ia = hand1.getAllFingers().keyList().iterator();
		
		BidiSortedMap< Long, Interval> fingers = hand1.getAllFingers();
		
		if (fingers.size() > 0) {
			//***** write the file name ******
			rabin.pushBytes(file.getName().getBytes());
			out.writeInt((int)rabin.getFingerprintLong());
			
			String fileExtension = (file.getName().indexOf(".") != -1)? file.getName().substring(file.getName().lastIndexOf(".") +1):"";
			fileExtension += "000";
			fileExtension = fileExtension.substring(0, 3);
			for (int i = 0; i < fileExtension.length(); i++) {
				out.writeChar(fileExtension.charAt(i));
			//	System.out.println((fileExtension.charAt(i)));
			}
			
			
			//****write the num of chunks
			//out.writeInt(fingers.size());
		//	System.out.println(fingers.size());
			if(fingers.size() >= 255){
				out.writeByte((byte)255);
				out.writeInt(fingers.size());
			 }
			 else
				 out.writeByte((byte)fingers.size());
					
	//		MainProgram.out_debug.write((int)rabin.getFingerprintLong() + " " + fingers.size() + "\n");
			
			
			byte [] data = new byte [2];
			//**outputs the chunk finger print and chunk size
			for (Map.Entry<Long, Interval> entry : fingers.entrySet()) {
				try {
					out.writeLong(entry.getKey());
					int chunkSize = entry.getValue().getSize().intValue();
					if(chunkSize < 65535){
						data[0] = (byte) (chunkSize & 0xFF);
						data[1] = (byte) ((chunkSize >> 8) & 0xFF);
						out.writeByte(data[1]);
						out.writeByte(data[0]);
					}
					else{
						out.writeByte(0xFF);
						out.writeByte(0xFF);
						out.writeInt(chunkSize);
					}

	//				MainProgram.out_debug.write(entry.getKey() + " "+ chunkSize + "\n");
			//		System.out.println(entry.getKey() + " "+ chunkSize);
				} catch (Exception e) {
					BLLogger.logData(e.getMessage());
				}
			}
			
//			MainProgram.out_debug.flush();

		}
		
	}
}



//public void calcFingerPrint(String basePath, BufferedWriter out) {
//	File directory = new File(basePath);
//	try{
//	for (File file : directory.listFiles()) {
//		if (file.isDirectory()) {
//			if(file.getAbsoluteFile().equals(file.getCanonicalFile()))
//				calcFingerPrint(file.getPath(), out);
//		} else if (file.isFile()) {
//			//files.add(file);
//			try{
//				Fingerprint<Polynomial> rabin = new RabinFingerprintLong(polynomial);
//				rabin.pushBytes(file.getName().getBytes());
//				out.write("@" + rabin.getFingerprint().toHexString());
//				getFingerPrints(file);
//				
//			}catch(Exception ex){
//				ex.printStackTrace();
//			}
//		}
//	}
//	}catch (Exception e) {
//		e.printStackTrace();
//	}
//}




